/**
 * Copyright (c) 2017-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the license found in the
 * LICENSE file in the root directory of this source tree.
 * @flow
 */
import {AbstractCrudObject} from './../abstract-crud-object';
import AbstractObject from './../abstract-object';
import AssignedUser from './assigned-user';
import CustomAudience from './custom-audience';
import DACheck from './da-check';
import Business from './business';
import AdAccount from './ad-account';
import AdsPixelStatsResult from './ads-pixel-stats-result';

/**
 * AdsPixel
 * @extends AbstractCrudObject
 * @see {@link https://developers.facebook.com/docs/marketing-api/}
 */
export default class AdsPixel extends AbstractCrudObject {
  static get Fields () {
    return Object.freeze({
      automatic_matching_fields: 'automatic_matching_fields',
      can_proxy: 'can_proxy',
      code: 'code',
      creation_time: 'creation_time',
      creator: 'creator',
      data_use_setting: 'data_use_setting',
      enable_automatic_matching: 'enable_automatic_matching',
      first_party_cookie_status: 'first_party_cookie_status',
      id: 'id',
      is_created_by_business: 'is_created_by_business',
      last_fired_time: 'last_fired_time',
      name: 'name',
      owner_ad_account: 'owner_ad_account',
      owner_business: 'owner_business'
    });
  }

  static get SortBy (): Object {
    return Object.freeze({
      last_fired_time: 'LAST_FIRED_TIME',
      name: 'NAME'
    });
  }
  static get AutomaticMatchingFields (): Object {
    return Object.freeze({
      em: 'em',
      fn: 'fn',
      ln: 'ln',
      ph: 'ph',
      ge: 'ge',
      zp: 'zp',
      ct: 'ct',
      st: 'st'
    });
  }
  static get DataUseSetting (): Object {
    return Object.freeze({
      empty: 'EMPTY',
      advertising_and_analytics: 'ADVERTISING_AND_ANALYTICS',
      analytics_only: 'ANALYTICS_ONLY'
    });
  }
  static get FirstPartyCookieStatus (): Object {
    return Object.freeze({
      empty: 'EMPTY',
      first_party_cookie_enabled: 'FIRST_PARTY_COOKIE_ENABLED',
      first_party_cookie_disabled: 'FIRST_PARTY_COOKIE_DISABLED'
    });
  }
  static get Tasks (): Object {
    return Object.freeze({
      edit: 'EDIT',
      analyze: 'ANALYZE'
    });
  }
  static get Type (): Object {
    return Object.freeze({
      primary: 'PRIMARY',
      secondary: 'SECONDARY'
    });
  }

  deleteAssignedUsers (params): AbstractObject {
    return super.deleteEdge(
      '/assigned_users',
      params
    );
  }

  getAssignedUsers (fields, params, fetchFirstPage = true): AssignedUser {
    return this.getEdge(
      AssignedUser,
      fields,
      params,
      fetchFirstPage,
      '/assigned_users'
    );
  }

  createAssignedUser (fields, params): AdsPixel {
    return this.createEdge(
      '/assigned_users',
      fields,
      params,
      AdsPixel
    );
  }

  getAudiences (fields, params, fetchFirstPage = true): CustomAudience {
    return this.getEdge(
      CustomAudience,
      fields,
      params,
      fetchFirstPage,
      '/audiences'
    );
  }

  createCreateServerToServerKey (fields, params): AdsPixel {
    return this.createEdge(
      '/create_server_to_server_keys',
      fields,
      params,
      AdsPixel
    );
  }

  getDaChecks (fields, params, fetchFirstPage = true): DACheck {
    return this.getEdge(
      DACheck,
      fields,
      params,
      fetchFirstPage,
      '/da_checks'
    );
  }

  getPendingSharedAgencies (fields, params, fetchFirstPage = true): Business {
    return this.getEdge(
      Business,
      fields,
      params,
      fetchFirstPage,
      '/pending_shared_agencies'
    );
  }

  createResetServerToServerKey (fields, params): AdsPixel {
    return this.createEdge(
      '/reset_server_to_server_key',
      fields,
      params,
      AdsPixel
    );
  }

  deleteSharedAccounts (params): AbstractObject {
    return super.deleteEdge(
      '/shared_accounts',
      params
    );
  }

  getSharedAccounts (fields, params, fetchFirstPage = true): AdAccount {
    return this.getEdge(
      AdAccount,
      fields,
      params,
      fetchFirstPage,
      '/shared_accounts'
    );
  }

  createSharedAccount (fields, params): AdsPixel {
    return this.createEdge(
      '/shared_accounts',
      fields,
      params,
      AdsPixel
    );
  }

  deleteSharedAgencies (params): AbstractObject {
    return super.deleteEdge(
      '/shared_agencies',
      params
    );
  }

  getSharedAgencies (fields, params, fetchFirstPage = true): Business {
    return this.getEdge(
      Business,
      fields,
      params,
      fetchFirstPage,
      '/shared_agencies'
    );
  }

  getStats (fields, params, fetchFirstPage = true): AdsPixelStatsResult {
    return this.getEdge(
      AdsPixelStatsResult,
      fields,
      params,
      fetchFirstPage,
      '/stats'
    );
  }

  get (fields, params): AdsPixel {
    return this.read(
      fields,
      params
    );
  }

  update (fields, params): AdsPixel {
    return super.update(
      params
    );
  }
}
